/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * Copyright 2008 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: JmolEmbeddedObject.java,v $
 * $Revision: 1.1 $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

package org.openoffice.embed.jmol;

import com.sun.star.embed.XStorage;
import com.sun.star.io.IOException;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.uno.AnyConverter;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.io.XStream;
import com.sun.star.io.XOutputStream;
import com.sun.star.io.XInputStream;
import com.sun.star.io.XTruncate;
import com.sun.star.beans.XPropertySet;
import com.sun.star.lang.XComponent;
import com.sun.star.container.XNameAccess;

import java.util.Vector;
import java.awt.Dimension;
import java.io.ByteArrayInputStream;
import java.nio.ByteBuffer;
import java.util.Properties;

/**
 *
 * @author ariel
 */
public final class JmolEmbeddedObject

        extends     WeakBase
        implements  com.sun.star.embed.XEmbedPersist,
                    com.sun.star.embed.XEmbeddedObject
{
    

    protected final XComponentContext m_xContext;
    protected final byte[] m_aClassID;
    protected boolean m_bDisposed = false;
    protected int m_nObjectState = -1;
    protected com.sun.star.embed.XStorage m_xParentStorage;
    protected com.sun.star.embed.XStorage m_xOwnStorage;
    protected String m_aEntryName;
    protected com.sun.star.embed.XStorage m_xNewParentStorage;
    protected com.sun.star.embed.XStorage m_xNewOwnStorage;
    protected String m_aNewEntryName;
    protected boolean m_bWaitSaveCompleted = false;
    protected JmolFrame m_aJmolFrame;
    protected Vector m_aListeners;
    com.sun.star.embed.VerbDescriptor[] m_pOwnVerbs;
    com.sun.star.embed.XEmbeddedClient m_xClient;
    Dimension m_aObjSize;


    // -------------------------------------------------------------

    public JmolEmbeddedObject(XComponentContext context, byte[] aClassID) {
        m_xContext = context;
        m_aClassID = aClassID;
    }


    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.embed.XEmbedPersist
    //
    // Description
    // specifies an implementation for embedded object persistence.
    // The idea is that any usable embedded object should be initialized
    // with an entry in the parent storage that will be used as persistent representation.
    //
    // inherits
    // com.sun.star.embed.XCommonEmbedPersist
    //
    // Description
    // specifies common implementation for embedded objects and links persistence.

    /**
     * com.sun.star.embed.XCommonEmbedPersist.storeOwn()
     *
     * lets the object or the link store itself.
     *
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.io.IOException   in case of io problems during saving
     * @throws com.sun.star.uno.Exception   in case of other problems
     */
    public void storeOwn()
            throws  com.sun.star.embed.WrongStateException,
                    com.sun.star.io.IOException,
                    com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1 || m_bWaitSaveCompleted) {
            throw new com.sun.star.embed.WrongStateException();
        }

        // nothing to do, if the object is in loaded state
        if (m_nObjectState == com.sun.star.embed.EmbedStates.LOADED) {
            return;
        }

        if (m_xOwnStorage == null) {
            throw new com.sun.star.io.IOException();
        }

        PostEvent("OnSave");

        if (m_aJmolFrame == null) {
            throw new com.sun.star.uno.RuntimeException();
        }

        SaveDataToStorage(m_xOwnStorage, m_aJmolFrame.getProperties());
                //m_aJmolFrame.getPDBString(), UpdateSizeAndGetFromActive());

        PostEvent("OnSaveDone");
    }

    /**
     * com.sun.star.embed.XCommonEmbedPersist.isReadonly()
     *
     * allows to detect if the data store is read-only.
     *
     * @return true   if the data store is readonly or opened readonly false otherwise
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state ( has no entry )
     */
    public boolean isReadonly() throws com.sun.star.embed.WrongStateException {
        return false;
    }

    /**
     * com.sun.star.embed.XCommonEmbedPersist.reload()
     *
     * lets the object or the link reload itself.
     *
     * If the object has persistance it will be reloaded from it's persistant entry.
     *
     * @param aMediaArgs    optional parameters for document reloading,
     *                      see also com.sun.star.document.MediaDescriptor
     * @param aObjectArgs   optional parameters for object reloading,
     *                      see also EmbeddedObjectDescriptor
     *
     * @throws com.sun.star.lang.IllegalArgumentException   one of arguments is illegal
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.io.IOException   in case of io problems during opening\creation
     * @throws com.sun.star.uno.Exception   in case of other problems
     */
    public void reload( com.sun.star.beans.PropertyValue[] aMediaArgs,
                        com.sun.star.beans.PropertyValue[] aObjectArgs)
            throws  com.sun.star.lang.IllegalArgumentException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.io.IOException,
                    com.sun.star.uno.Exception {
        // not implemented currently
        return;
    }


    /**
     * com.sun.star.embed.XEmbedPersist.setPersistentEntry()
     *
     * provides object with a parent storage and a name for object's entry.
     *
     * An entry with the specified name should be created/opened inside the
     * provided storage. It can be a storage or a stream.
     * For example, OOo API will refer to OLE storages only by streams,
     * but the object implementation will use storage based on this stream.
     *
     * Factory does this call to initialize the embedded object.
     * The linked object can be initialized by factory in a different way
     * ( internally ).
     * It is also possible to switch object persistent representation through
     * this call.
     * Actually this is the way, this call can be used by user ( since
     * initialization is done by factory ).
     *
     * @param xStorage  a parent storage the entry should be created in
     * @param sEntName  a name for the entry
     * @param nEntryConnectionMode    a mode in which the object should be
     *                                initialized from entry can take values
     *                                from EntryInitModes constant set
     * @param aMediaArgs    optional parameters for the embedded document
     *                      persistence initialization,
     *                      see also com.sun.star.document.MediaDescriptor
     * @param aObjectArgs   optional parameters for the object persistence
     *                      initialization, see also EmbeddedObjectDescriptor
     *
     * @throws com.sun.star.lang.IllegalArgumentException   one of arguments is illegal
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.io.IOException   in case of io problems during opening\creation
     * @throws com.sun.star.uno.Exception   in case of other problems
     */
    public void setPersistentEntry( com.sun.star.embed.XStorage xStorage,
                                    String aEntryName, int nEntryConnectionMode,
                                    com.sun.star.beans.PropertyValue[] aMediaArgs,
                                    com.sun.star.beans.PropertyValue[] aObjectArgs)
            throws  com.sun.star.lang.IllegalArgumentException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.io.IOException,
                    com.sun.star.uno.Exception {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (xStorage == null || aEntryName.length() == 0) {
            throw new com.sun.star.lang.IllegalArgumentException();
        }

        if ((m_nObjectState != -1 || nEntryConnectionMode ==
                com.sun.star.embed.EntryInitModes.NO_INIT) &&
                (m_nObjectState == -1 ||
                nEntryConnectionMode != com.sun.star.embed.EntryInitModes.NO_INIT)) {
            // if the object is not loaded
            // it can not get persistant representation without initialization

            // if the object is loaded
            // it can switch persistant representation only without initialization

            throw new com.sun.star.embed.WrongStateException();
        }

        if (m_bWaitSaveCompleted) {
            if (nEntryConnectionMode == com.sun.star.embed.EntryInitModes.NO_INIT) {
                if (m_xParentStorage == xStorage && m_aEntryName.equals(aEntryName)) {
                    saveCompleted(false);
                } else if (m_xNewParentStorage == xStorage && m_aNewEntryName.equals(aEntryName)) {
                    saveCompleted(true);
                } else {
                    throw new com.sun.star.embed.WrongStateException();
                }
            } else {
                throw new com.sun.star.embed.WrongStateException();
            }

            return;
        }

        boolean bElExists = xStorage.hasByName(aEntryName);

        if (nEntryConnectionMode == com.sun.star.embed.EntryInitModes.DEFAULT_INIT) {
            SwitchOwnPersistence(xStorage, aEntryName);
            if (bElExists) {
                XPropertySet xPropSet = (XPropertySet) UnoRuntime.queryInterface(
                        XPropertySet.class, m_xOwnStorage);
                if (xPropSet == null) {
                    throw new com.sun.star.uno.RuntimeException();
                }

                String aMediaType = AnyConverter.toString(xPropSet.getPropertyValue("MediaType"));
                if (!aMediaType.equals("application/x-openoffice-embedded-" +
                        JmolConstants.JMOL_EMBEDDED_OBJECT_CLSID)) {
                    throw new com.sun.star.lang.IllegalArgumentException();
                }

                m_nObjectState = com.sun.star.embed.EmbedStates.LOADED;
            } else {
                m_aJmolFrame = new JmolFrame(m_aEntryName, this, /*200, 200,*/ null);
                m_nObjectState = com.sun.star.embed.EmbedStates.RUNNING;
                m_aObjSize = m_aJmolFrame.getAppSize();
            }
        } else if (nEntryConnectionMode == com.sun.star.embed.EntryInitModes.TRUNCATE_INIT) {
            SwitchOwnPersistence(xStorage, aEntryName);
            m_aJmolFrame = new JmolFrame(m_aEntryName, this, /*200, 200,*/ null);
            m_nObjectState = com.sun.star.embed.EmbedStates.RUNNING;
            m_aObjSize = m_aJmolFrame.getAppSize();
        } else {
            throw new com.sun.star.lang.IllegalArgumentException();
        }
    }

    /**
     * com.sun.star.embed.XEmbedPersist.storeToEntry()
     *
     * lets the object store itself to an entry in destination storage,
     * the own persistence entry is not changed.
     * 
     * @param xStorage   a parent storage the entry should be created inside
     * @param sEntName   a name for the entry
     * @param aMediaArgs optional parameters for document saving,
     *                   see also com.sun.star.document.MediaDescriptor
     * @param aObjectArgs optional parameters for the object saving,
     *                    see also EmbeddedObjectDescriptor
     * 
     * @throws com.sun.star.lang.IllegalArgumentException   one of arguments is illegal
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.io.IOException   in case of io problems during storing
     * @throws com.sun.star.uno.Exception   in case of other problems
     */
    public void storeToEntry(   com.sun.star.embed.XStorage xStorage,
                                String aEntryName,
                                com.sun.star.beans.PropertyValue[] aMediaArgs,
                                com.sun.star.beans.PropertyValue[] aObjectArgs)
            throws  com.sun.star.lang.IllegalArgumentException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.io.IOException,
                    com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1 || m_bWaitSaveCompleted) {
            throw new com.sun.star.embed.WrongStateException();
        }

        if (m_nObjectState == com.sun.star.embed.EmbedStates.LOADED) {
            m_xParentStorage.copyElementTo(m_aEntryName, xStorage, aEntryName);
        } else {
            com.sun.star.embed.XStorage xSubStorage =
                    xStorage.openStorageElement(aEntryName,
                    com.sun.star.embed.ElementModes.READWRITE |
                    com.sun.star.embed.ElementModes.TRUNCATE);

            String aContents = m_aJmolFrame.getPDBString();

            SaveDataToStorage(xSubStorage, m_aJmolFrame.getProperties());
                //m_aJmolFrame.getPDBString(), UpdateSizeAndGetFromActive());
        }
    }

    /**
     * com.sun.star.embed.XEmbedPersist.storeAsEntry()
     *
     * lets the object store itself to an entry in destination storage
     * and prepare to use the new entry for own persistence.
     *
     * The object should be stored to the new entry, after that
     * the entry should be remembered by the object.
     * After the storing process is finished the XEmbedPersist.saveCompleted
     * method can be used to specify whether the object should use the new entry
     * or the old one.
     * The object persistence can not be used until XEmbedPersist.saveCompleted
     * is called. So this state can be treated as "HandsOff" state.
     * 
     * @param xStorage  a parent storage the entry should be created in
     * @param sEntName  a name for the entry
     * @param aMediaArgs optional parameters for document saving,
     *                   see also com.sun.star.document.MediaDescriptor
     * @param aObjectArgs optional parameters for the object saving,
     *                    see also EmbeddedObjectDescriptor
     * 
     * @thrown com.sun.star.lang.IllegalArgumentException   one of arguments is illegal
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.io.IOException   in case of io problems during storing
     * @throws com.sun.star.uno.Exception   in case of other problems
     */
    public void storeAsEntry(com.sun.star.embed.XStorage xStorage,
            String aEntryName, com.sun.star.beans.PropertyValue[] aMediaArgs,
            com.sun.star.beans.PropertyValue[] aObjectArgs)
            throws  com.sun.star.lang.IllegalArgumentException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.io.IOException,
                    com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1 || m_bWaitSaveCompleted) {
            throw new com.sun.star.embed.WrongStateException();
        }

        com.sun.star.embed.XStorage xSubStorage = null;

        if (m_nObjectState == com.sun.star.embed.EmbedStates.LOADED) {
            xSubStorage =
                    xStorage.openStorageElement(aEntryName,
                    com.sun.star.embed.ElementModes.READWRITE |
                    com.sun.star.embed.ElementModes.NOCREATE);

            m_xOwnStorage.copyToStorage(xSubStorage);
        } else {
            xSubStorage =
                    xStorage.openStorageElement(aEntryName,
                    com.sun.star.embed.ElementModes.READWRITE |
                    com.sun.star.embed.ElementModes.TRUNCATE);

            SaveDataToStorage(xSubStorage, m_aJmolFrame.getProperties());
                //m_aJmolFrame.getPDBString(), UpdateSizeAndGetFromActive());
        }

        m_bWaitSaveCompleted = true;
        m_xNewOwnStorage = xSubStorage;
        m_xNewParentStorage = xStorage;
        m_aNewEntryName = aEntryName;

    }

    /**
     * com.sun.star.embed.XEmbedPersist.saveCompleted()
     *
     * specifies whether the object should use an old storage or a new one
     * after "save as" operation.
     * 
     * @param bUseNew  true the new storage should be used false the old one
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.uno.Exception   in case of other problems
     */
   public void saveCompleted(boolean bUseNew)
            throws com.sun.star.embed.WrongStateException, com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        // it is allowed to call saveCompleted( false ) for nonstored objects
        if (!m_bWaitSaveCompleted && !bUseNew) {
            return;
        }

        if (!m_bWaitSaveCompleted) {
            throw new com.sun.star.io.IOException();
        }

        if (bUseNew) {
            SwitchOwnPersistence(m_xNewParentStorage, m_xNewOwnStorage, m_aNewEntryName);
            PostEvent("OnSaveAsDone");
        } else {
            try {
                m_xNewOwnStorage.dispose();
            } catch (com.sun.star.uno.RuntimeException e) {
            }
        }

        m_xNewOwnStorage = null;
        m_xNewParentStorage = null;
        m_aNewEntryName = null;
        m_bWaitSaveCompleted = false;
    }

    /**
     * com.sun.star.embed.XEmbedPersist.hasEntry()
     *
     * allows to detect if the object has entry.
     * 
     * @return true   if the object has own entry set false otherwise
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state ( has no entry )
     */
    public boolean hasEntry() throws com.sun.star.embed.WrongStateException {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_bWaitSaveCompleted) {
            throw new com.sun.star.embed.WrongStateException();
        }

        if (m_xOwnStorage != null) {
            return true;
        }

        return false;
    }

    /**
     * com.sun.star.embed.XEmbedPersist.getEntryName()
     *
     * allows to retrieve the current object entry name.
     * 
     * @return the object entry name if any
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state ( has no entry )
     */
    public String getEntryName() throws com.sun.star.embed.WrongStateException {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1 || m_bWaitSaveCompleted) {
            throw new com.sun.star.embed.WrongStateException();
        }

        return m_aEntryName;
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.embed.XVisualObject:
    //
    // represents common visualisation functionality for embedded objects.

    /**
     * com.sun.star.embed.XVisualObject.setVisualAreaSize()
     *
     * sets the size of object's visual area.
     * 
     * The size must be provided in logical units according to map mode 
     * the object communicates in.
     * If an object is inplace- or ui-active the method must not initiate 
     * repainting itself.
     * 
     * @param nAspect  the aspect specifying the form of object representation 
     *                 can take values from Aspects constant set
     * @param aSize    the new size of the visual area
     * 
     * @throws com.sun.star.lang.IllegalArgumentException   one of arguments is illegal
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.uno.Exception   the object failed to resize
     */
    public void setVisualAreaSize(long nAspect, com.sun.star.awt.Size aSize)
            throws  com.sun.star.lang.IllegalArgumentException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }
        // the ICON aspect should be handled by the container
        if (nAspect == com.sun.star.embed.Aspects.MSOLE_ICON) {
            throw new com.sun.star.embed.WrongStateException();
        }

        if (m_nObjectState == com.sun.star.embed.EmbedStates.LOADED) {
            changeState(com.sun.star.embed.EmbedStates.RUNNING);
        }

        if (m_aJmolFrame == null) {
            throw new com.sun.star.uno.RuntimeException();
        }

        m_aObjSize.setSize(aSize.Width, aSize.Height);
        m_aJmolFrame.setAppSize(m_aObjSize);
    }

    /**
     * com.sun.star.embed.XVisualObject.getVisualAreaSize()
     *
     * gets the size of object's visual area.
     * 
     * The size must be provided in logical units according to map mode 
     * the object communicates in.
     * 
     * @param nAspect   the aspect specifying the form of object representation 
     *                  can take values from Aspects constant set
     * @return  the size of visual area 
     * @throws com.sun.star.lang.IllegalArgumentException   one of arguments is illegal
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     */
    public com.sun.star.awt.Size getVisualAreaSize(long nAspect)
            throws  com.sun.star.lang.IllegalArgumentException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }
         // the ICON aspect should be handled by the container
        if (nAspect == com.sun.star.embed.Aspects.MSOLE_ICON) {
            throw new com.sun.star.embed.WrongStateException();
        }

        if (m_nObjectState == com.sun.star.embed.EmbedStates.LOADED) {
            changeState(com.sun.star.embed.EmbedStates.RUNNING);
        }

        UpdateSizeAndGetFromActive();

        return new com.sun.star.awt.Size(
                (int) m_aObjSize.getWidth(),
                (int) m_aObjSize.getHeight());
    }

    /**
     * com.sun.star.embed.XVisualObject.getPreferredVisualRepresentation()
     *
     * retrieves visual representation of the object in preferable format.
     * 
     * If the object persistance entry contains cached visual representation
     * then it can be retrieved by using this method even in loaded state.
     * 
     * @param nAspect the aspect the representation is requested for
     * @return the visual representation of the object in the default format
     * @throws com.sun.star.lang.IllegalArgumentException   one of arguments is illegal
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.uno.Exception   in case of problems
     */
    public com.sun.star.embed.VisualRepresentation getPreferredVisualRepresentation(long nAspect)
            throws  com.sun.star.lang.IllegalArgumentException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        // the ICON aspect should be handled by the container
        if (nAspect == com.sun.star.embed.Aspects.MSOLE_ICON) {
            throw new com.sun.star.embed.WrongStateException();
        }

        if (m_nObjectState == com.sun.star.embed.EmbedStates.LOADED) {
            changeState(com.sun.star.embed.EmbedStates.RUNNING);
        }

        byte[] aData = m_aJmolFrame.getReplacementImage();
        com.sun.star.embed.VisualRepresentation aVisRep = new com.sun.star.embed.VisualRepresentation();
        aVisRep.Data = aData;
        aVisRep.Flavor = new com.sun.star.datatransfer.DataFlavor(
                "image/png", "png", new com.sun.star.uno.Type(byte[].class));
        return aVisRep;
    }

    /**
     * com.sun.star.embed.XVisualObject.getMapUnit()
     *
     * retrieves map mode the object communicates in.
     * 
     * @param nAspect  the aspect the map mode is requested for
     * @return the map mode the object communicates in,
     *         it can take values from EmbedMapUnits constant
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     * @throws com.sun.star.uno.Exception  in case of problems
     */
    public int getMapUnit(long nAspect) throws com.sun.star.uno.Exception {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        // the ICON aspect should be handled by the container
        if (nAspect == com.sun.star.embed.Aspects.MSOLE_ICON) {
            throw new com.sun.star.embed.WrongStateException();
        }

        return com.sun.star.embed.EmbedMapUnits.PIXEL;
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.embed.XClassifiedObject:
    //
    // represents common functionality for embedded objects
    //
    //

    /**
     * com.sun.star.embed.XClassifiedObject.getClassID()
     *
     * retrieves class ID of the object.
     * 
     * @return unique class ID of the object
     */
    public byte[] getClassID() {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        return m_aClassID;
    }

    /**
     * com.sun.star.embed.XClassifiedObject.getClassName()
     *
     * retrieves symbolic name for the object type to be used in UI.
     * 
     * @return the symbolic name for the object
     */
    public String getClassName() {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        return new String();
    }

    /**
     * com.sun.star.embed.XClassifiedObject.setClassInfo()
     *
     * sets the class ID and symbolic name to an object.
     * 
     * @param aClassID   the new class ID
     * @param sClassName   the new symbolic name
     * @throws com.sun.star.lang.NoSupportException    in case changing of class information is not allowed
     */
     public void setClassInfo(byte[] aClassID, String sClassName)
            throws com.sun.star.lang.NoSupportException {

        throw new com.sun.star.lang.NoSupportException();
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.embed.XComponentSupplier:
    //
    // provides access to a component.

    /**
     * com.sun.star.embed.XComponentSupplier.getComponent()
     *
     * allows to get access to a component.
     * The component may not support com.sun.star.lang.XComponent interface.
     * 
     * @return component representation
     */
    public com.sun.star.util.XCloseable getComponent() {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }
        // allows no access to the component
        return null;
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.embed.XStateChangeBroadcaster:
    //
    // broadcasts message in case embedded object object changes it's state.

    /**
     * adds the specified listener to receive events about states change
     *
     * @param xListener   listener which is interested on such events
     */
    public void addStateChangeListener(
            com.sun.star.embed.XStateChangeListener xListener) {
        if (m_bDisposed) {
            return;
        }
        GetListeners().add(xListener);
    }

    /**
     * removes the specified listener
     *
     * @param Listener  listener which isn't interested on such events any longer
     */
    public void removeStateChangeListener(
            com.sun.star.embed.XStateChangeListener xListener) {
        if (m_bDisposed) {
            return;
        }

        if (m_aListeners != null) {
            m_aListeners.remove(xListener);
        }
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.document.XEventBroadcaster:
    //
    // makes it possible to register listeners which are called whenever a
    // document event (see EventObject) occurs

    /**
     * registers the given listener
     * 
     * @param xListener  listener which is interested on such events
     */
    public void addEventListener(com.sun.star.document.XEventListener xListener) {
        if (m_bDisposed) {
            return;
        }

        GetListeners().add(xListener);
    }

    /**
     * unregisters the given listener
     * 
     * @param x Listener  listener which isn't interested on such events any longer
     */
    public void removeEventListener(com.sun.star.document.XEventListener xListener) {
        if (m_bDisposed) {
            return;
        }

        if (m_aListeners != null) {
            m_aListeners.remove(xListener);
        }
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.util.XCloseBroadcaster:
    //
    // broadcasts each tried closing of an object to all interest listener
    //
    // The called object for closing must post the closing events immediately
    // and before any internal cancel operations will be started.
    // If a listener disagree with that it should throw a CloseVetoException and
    // called function XCloseable.close must be breaked immediately.
    // It's not allowed to catch it inside the close() request.
    // If no listener nor internal processes hinder the object on closing
    // all listener get's a notification about real closing.

    /**
     * adds the specified listener to receive or have a veto for "close" events
     * 
     * @param xListener  the listener wich is interest on closing events
     */
    public void addCloseListener(com.sun.star.util.XCloseListener xListener) {
        if (m_bDisposed) {
            return;
        }

        GetListeners().add(xListener);
    }

    /**
     * removes the specified listener
     * 
     * @param xListener  the listener wich isn't interest on closing events anylonger
     */
    public void removeCloseListener(com.sun.star.util.XCloseListener xListener) {
        if (m_bDisposed) {
            return;
        }

        if (m_aListeners != null) {
            m_aListeners.remove(xListener);
        }
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.util.XCloseable:
    //
    // makes it possible to release any objects in a ordered manner
    // by using a two-step mechanism
    //
    // If an object should be terminated, it can be:
    //
    //   * disposed (if it supports com.sun.star.lang.XComponent.dispose())
    //   * closed (if it supports close())
    //
    // First version gives the object no chance to disagree with that
    // (e.g. if a process is still running and can't be cancelled realy).
    // Last version provides this possibility, but can't guarantee
    // real termination of called object. It depends from the environment of
    // an object, if one or both mechanism are neccessary.
    //
    // Base interface XCloseBroadcaster makes it possible that any listener
    // which is interrested on life time of listened object ...
    //
    //   * can get a notification about closing of it
    //   * or can have a veto to break that.

    /**
     * try to close the object
     * 
     * Must definitly be called before com.sun.star.lang.XComponent.dispose().
     * But nobody can guarentee real closing of called object - because it can
     * disagree with that if any still running processes can't be cancelled yet.
     *
     * It's not allowed to block this call till internal operations will be
     * finished here. They must be cancelled or call must return imediatly by
     * throwing the CloseVetoException.
     * Otherwise (if nothing exist to disagree) it must return normaly.
     *
     * Before any internal processes will be cancelled, all registered
     * XCloseListener must be notified.
     * Any of them can disagree with a CloseVetoException too.
     * It's forbidden to catch this exception inside the called close() method
     * because the caller must get this information!
     * 
     * If somewhere disagree with a CloseVetoException it will not clear
     * who has to close the object again after still running processes was
     * finished. The parameter DeliverOwnership regulate that.
     * If it is set to false the caller of the method close() will be the owner
     * of this object in every case. Then it's not allowed to call close() from
     * any other place (may a registered XCloseListener).
     * If it is set to true the caller gives up his ownership.
     * If a XCloseListener throw the veto exception he will be the new owner of
     * the closing object.
     * This information is passed to the listener by a parameter of his
     * notification method XCloseListener.queryClosing().
     * After his operations was finished he MUST try to close it again.
     * If the closing object itselfs disagree by an exception and the parameter
     * DeliverOwnership was set to true the object will be his own owner with
     * all consequences of that.
     *
     * Note:
     * 
     * There is no way to get the ownership back if it was delivered!
     *
     * If this method was already called on an object it should return without
     * any reaction. Normaly it's possible to throw a
     * com.sun.star.lang.DisposedException for already disposed or closed objects
     * (which represent a com.sun.star.uno.RuntimeException and can be thrown
     * by every interface call), but it shouldn't be used here.
     * The veto exception should be the only way to indicates the result.
     * 
     * @param DeliverOwnership  true  delegates the ownership of ths closing object
     *                          to any one which throw the CloseVetoException.
     *                          This new owner has to close the closing object again
     *                          if his still running processes will be finished.
     *                          false let the ownership at the original one which
     *                          called the close() method. He must react for possible
     *                          CloseVetoExceptions and try it again at a later time.
     *                          This can be usefull for a generic UI handling.
     * 
     * @throws com.sun.star.util.CloseVetoException
     *                            indicates that the closing object himself or
     *                            any of his currently registered listener disagree
     *                            with this close() request.
     */
    public void close(boolean bDeliverOwnership) throws com.sun.star.util.CloseVetoException {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        com.sun.star.lang.EventObject aEventObject = new com.sun.star.lang.EventObject(this);

        if (m_aListeners != null) {
            for (int nInd = 0; nInd < m_aListeners.size(); nInd++) {
                try {
                    com.sun.star.util.XCloseListener xListener =
                            (com.sun.star.util.XCloseListener) UnoRuntime.queryInterface(
                            com.sun.star.document.XEventListener.class,
                            m_aListeners.get(nInd));

                    if (xListener != null) {
                        xListener.queryClosing(aEventObject, bDeliverOwnership);
                    }
                } catch (com.sun.star.util.CloseVetoException e) {
                    throw e;
                } catch (java.lang.Exception e) {
                    m_aListeners.remove(nInd);
                }
            }

            m_bDisposed = true;

            for (int nInd = 0; nInd < m_aListeners.size(); nInd++) {
                try {
                    com.sun.star.util.XCloseListener xListener =
                            (com.sun.star.util.XCloseListener) UnoRuntime.queryInterface(
                            com.sun.star.document.XEventListener.class,
                            m_aListeners.get(nInd));

                    if (xListener != null) {
                        xListener.notifyClosing(aEventObject);
                    }
                } catch (java.lang.Exception e) {
                    m_aListeners.remove(nInd);
                }
            }
            m_aListeners.clear();
        }
        m_bDisposed = true;

        if (m_aJmolFrame != null) {
            m_aJmolFrame.dispose();
            m_aJmolFrame = null;
        }

        if (m_xOwnStorage != null) {
            try {
                m_xOwnStorage.dispose();
            } catch (java.lang.Exception e) {
            }
            m_xOwnStorage = null;
        }
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    // com.sun.star.embed.XEmbeddedObject:
    //
    // represents common functionality for embedded objects

    /**
     * changes the state of the object to the requested one.
     * 
     * @param nNewState  specifies the new state,
     *                   can take values from the constant set EmbedStates
     *
     * @throws com.sun.star.embed.UnreachableStateException    the specified state can not be reached
     * @throws com.sun.star.embed.WrongStateException    in case object is in invalid state
     * @throws com.sun.star.uno.Exception    in case of other problems
     */
    public void changeState(int nNewState)
            throws  com.sun.star.embed.UnreachableStateException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        int nOldState = m_nObjectState;

        if (nOldState == nNewState) {
            if (nOldState == com.sun.star.embed.EmbedStates.ACTIVE) {
                if (m_aJmolFrame == null) {
                    throw new com.sun.star.uno.RuntimeException();
                }
                m_aJmolFrame.toFront();
            }

            return;
        }

        if (nNewState != com.sun.star.embed.EmbedStates.LOADED &&
                nNewState != com.sun.star.embed.EmbedStates.RUNNING &&
                nNewState != com.sun.star.embed.EmbedStates.ACTIVE)
        {
            throw new com.sun.star.embed.UnreachableStateException();
        }

        StateChangeNotification(true, nOldState, nNewState);

        try {
            if (nOldState == com.sun.star.embed.EmbedStates.LOADED) {
                // switch to the RUNNING state first

                java.util.Properties aProperties = UnoStreamToJavaProperties(
                        m_xOwnStorage,
                        JmolConstants.STREAM_ELEMENT_JMOL_PROPERTIES_FILE);
                /*String aText = ReadStringFromStream(
                        m_xOwnStorage, JmolConstants.STREAM_ELEMENT_PDB_FILE);*/

                JmolFrame aJmolFrame = new JmolFrame(m_aEntryName, this, /*200, 200,*/ aProperties);
                //aJmolFrame.setPDBString(aText);

                //ReadSizeFromOwnStorage();

                m_aJmolFrame = aJmolFrame;
                m_nObjectState = com.sun.star.embed.EmbedStates.RUNNING;

                if (nNewState == com.sun.star.embed.EmbedStates.ACTIVE) {
                    if (m_xClient == null) {
                        throw new com.sun.star.embed.WrongStateException();
                    }

                    m_aJmolFrame.setVisible(true);
                    m_aJmolFrame.toFront();

                    if (m_aObjSize != null) {
                        aJmolFrame.setAppSize(m_aObjSize);
                    }

                    m_xClient.visibilityChanged(true);
                    m_nObjectState = com.sun.star.embed.EmbedStates.ACTIVE;
                }
            } else if (nOldState == com.sun.star.embed.EmbedStates.RUNNING) {
                if (nNewState == com.sun.star.embed.EmbedStates.LOADED) {
                    JmolFrame aFrame = m_aJmolFrame;
                    m_aJmolFrame = null;
                    aFrame.dispose();
                    m_nObjectState = nNewState;
                } else // nNewState == ACTIVE
                {
                    if (m_aJmolFrame == null) {
                        throw new com.sun.star.uno.RuntimeException();
                    }

                    if (m_xClient == null) {
                        throw new com.sun.star.embed.WrongStateException();
                    }

                    m_aJmolFrame.setVisible(true);
                    m_aJmolFrame.toFront();

                    if (m_aObjSize != null) {
                        m_aJmolFrame.setAppSize(m_aObjSize);
                    }

                    m_xClient.visibilityChanged(true);

                    m_nObjectState = nNewState;
                }
            } else // nOldState == ACTIVE
            {
                UpdateSizeAndGetFromActive();
                if (nNewState == com.sun.star.embed.EmbedStates.RUNNING) {
                    m_aJmolFrame.setVisible(false);
                    m_nObjectState = nNewState;
                } else // nNewState == LOADED
                {
                    JmolFrame aFrame = m_aJmolFrame;
                    m_aJmolFrame = null;
                    aFrame.dispose();
                    m_nObjectState = nNewState;
                }
            }
        } catch (com.sun.star.uno.Exception e) {
            if (nOldState != m_nObjectState) {
                StateChangeNotification(false, nOldState, m_nObjectState);
            }
            throw e;
        }

        StateChangeNotification(true, nOldState, nNewState);
    }

    /**
     * returns supported states for the object.
     * 
     * @return  the sequence of states the object can be set to
     * 
     * @throws com.sun.star.embed.NeedsRunnignStateException
     *         means that the object is in loaded state now and can be switched
     *         to running state, other possible states can be detected only
     *         when the object is not in loaded state any more
     * 
     * @throws com.sun.star.embed.WrongStateException in case object is in invalid state
     */
    public int[] getReachableStates()
            throws  com.sun.star.embed.NeedsRunningStateException,
                    com.sun.star.embed.WrongStateException
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        int[] pStates = new int[3];
        pStates[0] = com.sun.star.embed.EmbedStates.LOADED;
        pStates[1] = com.sun.star.embed.EmbedStates.RUNNING;
        pStates[2] = com.sun.star.embed.EmbedStates.ACTIVE;

        return pStates;
    }

    /**
     * returns the current state of the object.
     * 
     * @return the current state of the object
     * @throws com.sun.star.embed.WrongStateException   in case object is in invalid state
     */
    public int getCurrentState() throws com.sun.star.embed.WrongStateException {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        return m_nObjectState;
    }

    /**
     * lets object perform an action referenced by nVerbID.
     * 
     * @param nVerbID specifies an action to perform,
     *                can take values from EmbedVerbs
     * 
     * @throws com.sun.star.lang.IllegalArgumentException   the verb is not supported
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state to call the function
     * @throws com.sun.star.embed.UnreachableStateException   the state, required by the verb, can not be reached
     * @throws com.sun.star.uno.Exception   in case of other problems
     */
    public void doVerb(int nVerbID)
            throws  com.sun.star.lang.IllegalArgumentException,
                    com.sun.star.embed.WrongStateException,
                    com.sun.star.embed.UnreachableStateException,
                    com.sun.star.uno.Exception
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        if (nVerbID == com.sun.star.embed.EmbedVerbs.MS_OLEVERB_PRIMARY ||
                nVerbID == com.sun.star.embed.EmbedVerbs.MS_OLEVERB_SHOW ||
                nVerbID == com.sun.star.embed.EmbedVerbs.MS_OLEVERB_OPEN)
        {
            changeState(com.sun.star.embed.EmbedStates.ACTIVE);
        } else if (nVerbID == com.sun.star.embed.EmbedVerbs.MS_OLEVERB_HIDE) {
            changeState(com.sun.star.embed.EmbedStates.RUNNING);
        }
    }

    /**
     * returns supported verbs for the object
     * 
     * @return the sequence of verbs the object supports
     * @throws com.sun.star.embed.NeedsRunnignStateException
     *         means that the object is in loaded state now and can be switched
     *         to running state, acceptable verbs can be detected only when the
     *         object is not in loaded state any more
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state to call the function
     */
    public com.sun.star.embed.VerbDescriptor[] getSupportedVerbs()
            throws  com.sun.star.embed.NeedsRunningStateException,
                    com.sun.star.embed.WrongStateException
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        if (m_pOwnVerbs == null) {
            try {
                XMultiComponentFactory xFactory = m_xContext.getServiceManager();
                Object obj = xFactory.createInstanceWithContext(
                        "com.sun.star.configuration.ConfigurationProvider", m_xContext);
                XMultiServiceFactory xConfProvider =
                        (XMultiServiceFactory) UnoRuntime.queryInterface(
                        XMultiServiceFactory.class, obj);
                if (xConfProvider == null) {
                    throw new com.sun.star.uno.RuntimeException();
                }

                Object[] aArgs = new Object[1];
                aArgs[0] = "/org.openoffice.Office.Embedding/Objects";
                Object oSettings = xConfProvider.createInstanceWithArguments(
                        "com.sun.star.configuration.ConfigurationAccess", aArgs);
                XNameAccess xObjConfNA = (XNameAccess) UnoRuntime.queryInterface(
                        XNameAccess.class, oSettings);
                if (xObjConfNA == null) {
                    throw new com.sun.star.uno.RuntimeException();
                }

                Object oEmbObj = xObjConfNA.getByName(
                        JmolConstants.JMOL_EMBEDDED_OBJECT_CLSID);
                XNameAccess xEmbObjNA = (XNameAccess) UnoRuntime.queryInterface(
                        XNameAccess.class, oEmbObj);
                if (xEmbObjNA == null) {
                    throw new com.sun.star.uno.RuntimeException();
                }

                String[] pVerbShortcuts = (String[]) AnyConverter.toArray(
                        xEmbObjNA.getByName("ObjectVerbs"));
                if (pVerbShortcuts != null && pVerbShortcuts.length != 0) {
                    com.sun.star.embed.VerbDescriptor[] pVerbs =
                            new com.sun.star.embed.VerbDescriptor[pVerbShortcuts.length];
                    aArgs[0] = "/org.openoffice.Office.Embedding/Verbs";
                    Object oVerbs = xConfProvider.createInstanceWithArguments(
                            "com.sun.star.configuration.ConfigurationAccess", aArgs);
                    XNameAccess xVerbsConfNA = (XNameAccess) UnoRuntime.queryInterface(
                            XNameAccess.class, oVerbs);
                    if (xVerbsConfNA == null) {
                        throw new com.sun.star.uno.RuntimeException();
                    }

                    for (int nInd = 0; nInd < pVerbShortcuts.length; nInd++) {
                        try {
                            XNameAccess xVerbNA = (XNameAccess) UnoRuntime.queryInterface(
                                    XNameAccess.class,
                                    xVerbsConfNA.getByName(pVerbShortcuts[nInd]));
                            if (xVerbNA != null) {
                                com.sun.star.embed.VerbDescriptor aVerb =
                                        new com.sun.star.embed.VerbDescriptor();
                                aVerb.VerbID = AnyConverter.toInt(xVerbNA.getByName("VerbID"));
                                aVerb.VerbName = AnyConverter.toString(xVerbNA.getByName("VerbUIName"));
                                aVerb.VerbFlags = AnyConverter.toInt(xVerbNA.getByName("VerbFlags"));
                                aVerb.VerbAttributes = AnyConverter.toInt(xVerbNA.getByName("VerbAttributes"));
                                pVerbs[nInd] = aVerb;
                            }
                        } catch (java.lang.Exception e) {
                        }

                        if (pVerbs[nInd] == null) {
                            // let the error be visible
                            pVerbs[nInd] = new com.sun.star.embed.VerbDescriptor();
                            pVerbs[nInd].VerbID = com.sun.star.embed.EmbedVerbs.MS_OLEVERB_PRIMARY;
                            pVerbs[nInd].VerbName = "ERROR!";
                            pVerbs[nInd].VerbFlags = 0;
                            pVerbs[nInd].VerbAttributes = com.sun.star.embed.VerbAttributes.MS_VERBATTR_ONCONTAINERMENU;
                        }
                    }

                    m_pOwnVerbs = pVerbs;
                }
            } catch (com.sun.star.uno.Exception e) {
            }
        }

        if (m_pOwnVerbs != null) {
            return m_pOwnVerbs;
        }

        return new com.sun.star.embed.VerbDescriptor[0];
    }

    /**
     * sets a connection to the container's client.
     * 
     * @param xClient  provides a reference to a client implementation
     * @throws com.sun.star.embed.WrongStateException  the object is in wrong state
     */
    public void setClientSite(com.sun.star.embed.XEmbeddedClient xClient)
            throws com.sun.star.embed.WrongStateException
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        m_xClient = xClient;
    }

    /**
     * provides access to the internal link to the container client.
     * 
     * @return a reference to related container client if any is set
     * @throws com.sun.star.embed.WrongStateException   the object is in wrong state
     */
    public com.sun.star.embed.XEmbeddedClient getClientSite()
            throws com.sun.star.embed.WrongStateException
    {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        return m_xClient;
    }

    /**
     * updates object's representations.
     * 
     * @throws com.sun.star.embed.WrongStateException the object is in wrong state to call the function
     * @throws com.sun.star.uno.Exception in case problems detected
     */
    public void update() throws com.sun.star.embed.WrongStateException,
            com.sun.star.uno.Exception {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

    // not implemented
    }

    /**
     * specifies how often the object's representation should be updated.
     * 
     * @param nMode the new update mode, can take values from EmbeddedUpdateModes
     * @throws com.sun.star.embed.WrongStateException the object is in wrong state
     */
    public void setUpdateMode(int nMode) throws com.sun.star.embed.WrongStateException {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

    // not implemented
    }

    /**
     * retrieves the status of the object.
     * 
     * @param nAspect the aspect specifying the form of object representation
     * @return the value specifying the status of the object for specified aspect
     *         can take values from EmbedMisc constant set
     * @throws com.sun.star.embed.WrongStateException the object is in wrong state
     */
    public long getStatus(long nAspect) throws com.sun.star.embed.WrongStateException {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

        if (m_nObjectState == -1) {
            throw new com.sun.star.embed.WrongStateException();
        }

        return 0;
    }

    /**
     * provides object with the name of container document.
     * 
     * @param sName name of the container document
     */
    public void setContainerName(String sName) {
        if (m_bDisposed) {
            throw new com.sun.star.lang.DisposedException();
        }

    // not implemented
    }

    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************
    //*************************************************************************

    

    // -------------------------------------------------------------
    protected Vector GetListeners() {
        if (m_aListeners == null) {
            m_aListeners = new Vector<Object>(10, 10);
        }

        return m_aListeners;
    }

    // -------------------------------------------------------------

    protected Dimension UpdateSizeAndGetFromActive() {
        if (m_nObjectState == com.sun.star.embed.EmbedStates.ACTIVE) {
            m_aObjSize = m_aJmolFrame.getAppSize();
        }

        if (m_aObjSize != null) {
            return m_aObjSize;
        } else {
            return new Dimension();
        }
    }

    /*protected void ReadSizeFromOwnStorage()
            throws com.sun.star.io.IOException {
        String aSize = ReadStringFromStream(m_xOwnStorage, JmolConstants.STREAM_ELEMENT_JMOL_PROPERTIES_FILE);

        int nSeparator = aSize.indexOf('|');
        if (nSeparator > 0 && nSeparator < aSize.length() - 1) {
            int nWidth = Integer.parseInt(aSize.substring(0, nSeparator));
            int nHeight = Integer.parseInt(
                    aSize.substring(nSeparator + 1, aSize.length()));
            m_aObjSize = new Dimension(nWidth, nHeight);
        }
    }*/



    // -------------------------------------------------------------
    /*protected String ReadStringFromStream(com.sun.star.embed.XStorage xStorage, String aStreamName)
            throws com.sun.star.io.IOException {
        if (xStorage == null) {
            throw new com.sun.star.uno.RuntimeException();
        }

        try {
            XStream xStream = xStorage.openStreamElement(aStreamName,
                    com.sun.star.embed.ElementModes.READWRITE);
            XComponent xStreamComp = (XComponent) UnoRuntime.queryInterface(
                    XComponent.class, xStream);
            if (xStreamComp == null) {
                throw new com.sun.star.uno.RuntimeException();
            }

            XInputStream xInStream = xStream.getInputStream();
            byte[][] aData = new byte[1][];
            aData[0] = new byte[0];
            String aResult = new String();

            int nLen = 0;
            do {
                nLen = xInStream.readBytes(aData, 10);
                if (aData.length == 0 || aData[0] == null) {
                    throw new com.sun.star.io.IOException();
                }
                aResult += new String(aData[0]);
            } while (nLen > 0);

            xStreamComp.dispose();

            return aResult;
        } catch (com.sun.star.uno.RuntimeException e) {
            throw e;
        } catch (com.sun.star.io.IOException e) {
            throw e;
        } catch (com.sun.star.uno.Exception e) {
            throw new com.sun.star.io.IOException("Error while reading one of object streams!");
        }
    }*/


    // -------------------------------------------------------------
    protected void SwitchOwnPersistence(
            com.sun.star.embed.XStorage xParentStorage, 
            com.sun.star.embed.XStorage xOwnStorage,
            String aEntryName) {
        if (xOwnStorage != m_xOwnStorage) {
            if (m_xOwnStorage != null) {
                m_xOwnStorage.dispose();
            }
            m_xParentStorage = xParentStorage;
            m_xOwnStorage = xOwnStorage;
            m_aEntryName = aEntryName;
        }
    }

    protected void SwitchOwnPersistence(
            com.sun.star.embed.XStorage xParentStorage,
            String aEntryName)
            throws com.sun.star.io.IOException {
        if (xParentStorage != m_xParentStorage || !aEntryName.equals(m_aEntryName)) {
            try {
                com.sun.star.embed.XStorage xOwnStorage = xParentStorage.openStorageElement(
                        aEntryName, com.sun.star.embed.ElementModes.READWRITE);
                SwitchOwnPersistence(xParentStorage, xOwnStorage, aEntryName);
            } catch (com.sun.star.uno.RuntimeException e) {
                throw e;
            } catch (com.sun.star.io.IOException e) {
                throw e;
            } catch (com.sun.star.uno.Exception e) {
                throw new com.sun.star.io.IOException("Error while switching object storage!");
            }
        }
    }

    // -------------------------------------------------------------
    protected static void SaveDataToStorage(
            com.sun.star.embed.XStorage xStorage,
            String aString,
            Dimension aDimension) throws com.sun.star.io.IOException {
        try {
            // save the text
            XStream xPDBStream = xStorage.openStreamElement(JmolConstants.STREAM_ELEMENT_PDB_FILE,
                    com.sun.star.embed.ElementModes.READWRITE);
            XComponent xPDBStreamComp = (XComponent) UnoRuntime.queryInterface(
                    XComponent.class, xPDBStream);
            if (xPDBStreamComp == null) {
                throw new com.sun.star.uno.RuntimeException();
            }

            XOutputStream xOutStream = xPDBStream.getOutputStream();
            XTruncate xTruncate = (XTruncate) UnoRuntime.queryInterface(
                    XTruncate.class, xOutStream);
            if (xTruncate == null) {
                throw new com.sun.star.io.IOException();
            }

            xTruncate.truncate();
            xOutStream.writeBytes(aString.getBytes());

            // set the media type
            XPropertySet xPropSet = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, xPDBStream);
            if (xPropSet != null) {
                xPropSet.setPropertyValue("MediaType", "application/binary");
            }

            // save the size
            XStream xPropsStream = xStorage.openStreamElement(JmolConstants.STREAM_ELEMENT_JMOL_PROPERTIES_FILE,
                    com.sun.star.embed.ElementModes.READWRITE);
            XComponent xPropsStreamComp = (XComponent) UnoRuntime.queryInterface(
                    XComponent.class, xPropsStream);
            if (xPropsStreamComp == null) {
                throw new com.sun.star.uno.RuntimeException();
            }

            xOutStream = xPropsStream.getOutputStream();
            xTruncate = (XTruncate) UnoRuntime.queryInterface(
                    XTruncate.class, xOutStream);
            if (xTruncate == null) {
                throw new com.sun.star.io.IOException();
            }

            xTruncate.truncate();
            String aProps = Integer.toString((int) aDimension.getWidth()) + "|" + Integer.toString((int) aDimension.getHeight());
            xOutStream.writeBytes(aProps.getBytes());

            // set the media type
            xPropSet = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, xPropsStream);
            if (xPropSet != null) {
                xPropSet.setPropertyValue("MediaType", "application/binary");
            }

            // set the media type
            xPropSet = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, xStorage);
            if (xPropSet == null) {
                throw new com.sun.star.uno.RuntimeException();
            }
            xPropSet.setPropertyValue("MediaType",
                    "application/x-openoffice-embedded-" +
                    JmolConstants.JMOL_EMBEDDED_OBJECT_CLSID);

            com.sun.star.embed.XTransactedObject xTransact = (com.sun.star.embed.XTransactedObject) UnoRuntime.queryInterface(
                    com.sun.star.embed.XTransactedObject.class, xStorage);
            if (xTransact != null) {
                xTransact.commit();
            }

            xPDBStreamComp.dispose();
            xPropsStreamComp.dispose();

        } catch (com.sun.star.uno.RuntimeException e) {
            throw e;
        } catch (com.sun.star.io.IOException e) {
            throw e;
        } catch (com.sun.star.uno.Exception e) {
            throw new com.sun.star.io.IOException(
                    "Error while switching object storage!");
        }
    }

    protected static void SaveDataToStorage(
            com.sun.star.embed.XStorage xStorage,
            java.util.Properties aProperties) throws com.sun.star.io.IOException {
        try {
            // save the text
            byte[] aPropertiesBytes = null;
            if (aProperties != null) {
                try {
                    java.io.File aTmpFile = java.io.File.createTempFile("temp_ooo_jmol", ".properties");
                    java.io.FileOutputStream aOutStream = new java.io.FileOutputStream(aTmpFile);
                    aProperties.store(aOutStream, "OpenOffice.org Jmol Embedded Object");

                    int nLen = (int) aTmpFile.length();
                    aPropertiesBytes = new byte[nLen];
                    java.io.FileInputStream aTmpStream = new java.io.FileInputStream(aTmpFile);
                    aTmpStream.read(aPropertiesBytes);
                    aTmpStream.close();
                    aTmpFile.delete();
                } catch (java.lang.Exception e) {
                }
            }


            XStream xPropsStream = xStorage.openStreamElement(
                    JmolConstants.STREAM_ELEMENT_JMOL_PROPERTIES_FILE,
                    com.sun.star.embed.ElementModes.READWRITE);
            XComponent xPropsStreamComp = (XComponent) UnoRuntime.queryInterface(
                    XComponent.class, xPropsStream);
            if (xPropsStreamComp == null) {
                throw new com.sun.star.uno.RuntimeException();
            }

            XOutputStream xOutStream = xPropsStream.getOutputStream();
            XTruncate xTruncate = (XTruncate) UnoRuntime.queryInterface(
                    XTruncate.class, xOutStream);
            if (xTruncate == null) {
                throw new com.sun.star.io.IOException();
            }

            xTruncate.truncate();
            xOutStream.writeBytes(aPropertiesBytes);

            // set the media type
            XPropertySet xPropSet = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, xPropsStream);
            if (xPropSet != null) {
                xPropSet.setPropertyValue("MediaType", "application/binary");
            }

            // set the media type
            xPropSet = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, xStorage);
            if (xPropSet == null) {
                throw new com.sun.star.uno.RuntimeException();
            }
            xPropSet.setPropertyValue("MediaType",
                    "application/x-openoffice-embedded-" +
                    JmolConstants.JMOL_EMBEDDED_OBJECT_CLSID);

            com.sun.star.embed.XTransactedObject xTransact =
                    (com.sun.star.embed.XTransactedObject) UnoRuntime.queryInterface(
                    com.sun.star.embed.XTransactedObject.class, xStorage);
            if (xTransact != null) {
                xTransact.commit();
            }

            xPropsStreamComp.dispose();

        } catch (com.sun.star.uno.RuntimeException e) {
            throw e;
        } catch (com.sun.star.io.IOException e) {
            throw e;
        } catch (com.sun.star.uno.Exception e) {
            throw new com.sun.star.io.IOException(
                    "Error while switching object storage!");
        }
    }

    // -------------------------------------------------------------
    protected void PostEvent(String aEvEntryName) {
        if (m_aListeners != null) {
            com.sun.star.document.EventObject aEventObject =
                    new com.sun.star.document.EventObject(this, aEvEntryName);

            for (int nInd = 0; nInd < m_aListeners.size(); nInd++) {
                try {
                    com.sun.star.document.XEventListener xListener =
                            (com.sun.star.document.XEventListener) UnoRuntime.queryInterface(
                            com.sun.star.document.XEventListener.class,
                            m_aListeners.get(nInd));

                    if (xListener != null) {
                        xListener.notifyEvent(aEventObject);
                    }
                } catch (java.lang.Exception e) {
                    m_aListeners.remove(nInd);
                }
            }
        }
    }

    // -------------------------------------------------------------
    protected void StateChangeNotification(
            boolean bBeforeChange, int nOldState, int nNewState) {
        if (m_aListeners != null) {
            com.sun.star.lang.EventObject aEventObject =
                    new com.sun.star.lang.EventObject(this);

            for (int nInd = 0; nInd < m_aListeners.size(); nInd++) {
                try {
                    com.sun.star.embed.XStateChangeListener xListener =
                            (com.sun.star.embed.XStateChangeListener) UnoRuntime.queryInterface(
                            com.sun.star.embed.XStateChangeListener.class,
                            m_aListeners.get(nInd));

                    if (xListener != null) {
                        if (bBeforeChange) {
                            xListener.changingState(aEventObject, nOldState, nNewState);
                        } else {
                            xListener.stateChanged(aEventObject, nOldState, nNewState);
                        }
                    }
                } catch (java.lang.Exception e) {
                    m_aListeners.remove(nInd);
                }
            }
        }
    }
    // -------------------------------------------------------------
    

    // -------------------------------------------------------------
    public void CloseFrameRequest() {
        com.sun.star.embed.XEmbeddedClient xClient = m_xClient;
        if (xClient == null) {
            return;
        }

        UpdateSizeAndGetFromActive();
        StateChangeNotification(true, com.sun.star.embed.EmbedStates.ACTIVE,
                com.sun.star.embed.EmbedStates.RUNNING);

        try {
            xClient.visibilityChanged(false);
        } catch (com.sun.star.uno.Exception e) {
        }

        try {
            xClient.saveObject();
        } catch (com.sun.star.uno.Exception e) {
        }

        m_nObjectState = com.sun.star.embed.EmbedStates.RUNNING;
        StateChangeNotification(false, com.sun.star.embed.EmbedStates.ACTIVE, m_nObjectState);

        PostEvent("OnVisAreaChanged");
    }

    private java.util.Properties UnoStreamToJavaProperties(
            XStorage xStorage,
            String sStorageElementName){
        java.util.Properties aProperties = null;
        try {
            if (xStorage != null) {

                XStream xStream = xStorage.openStreamElement(
                        sStorageElementName,
                        com.sun.star.embed.ElementModes.READWRITE);

                XComponent xStreamComp = (XComponent) UnoRuntime.queryInterface(
                        XComponent.class, xStream);

                if (xStreamComp != null) {

                    XInputStream xInStream = xStream.getInputStream();

                    byte[][] aData = new byte[1][];
                    aData[0] = new byte[0];
                    ByteBuffer buffer = ByteBuffer.allocate(xInStream.available());

                    int nLen = 0;
                    do {
                        nLen = xInStream.readBytes(aData, 10);
                        if (aData.length == 0 || aData[0] == null) {
                            throw new com.sun.star.io.IOException();
                        }
                        buffer.put(aData[0]);
                    } while (nLen > 0);

                    xStreamComp.dispose();
                    
                    ByteArrayInputStream in = new ByteArrayInputStream(buffer.array());
                    aProperties = new Properties();
                    aProperties.load(in);
                    in.close();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return aProperties;
        }
    }


}

